define(["require", "exports", "../utils/helper", "@syncfusion/ej2-base", "../utils/helper", "../model/constants"], function (require, exports, helper_1, ej2_base_1, helper_2, constants_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var LayoutPanel = (function () {
        function LayoutPanel(treemap) {
            this.treemap = treemap;
        }
        LayoutPanel.prototype.processLayoutPanel = function () {
            var data;
            var totalRect;
            if (this.treemap.levelsOfData && this.treemap.levelsOfData.length > 0) {
                data = (!ej2_base_1.isNullOrUndefined(this.treemap.initialDrillDown.groupIndex) &&
                    !ej2_base_1.isNullOrUndefined(this.treemap.initialDrillDown.groupName)) ?
                    this.getDrilldownData(this.treemap.levelsOfData[0], [])[0] : this.treemap.levelsOfData[0];
                totalRect = ej2_base_1.extend({}, this.treemap.areaRect, totalRect, false);
                this.calculateLayoutItems(data || this.treemap.levelsOfData[0], totalRect);
                this.renderLayoutItems(data || this.treemap.levelsOfData[0]);
            }
        };
        LayoutPanel.prototype.getDrilldownData = function (data, drillData) {
            var treemap = this.treemap;
            var newData = {};
            var child = helper_2.findChildren(data)['values'];
            if (child && child.length > 0 && drillData.length === 0) {
                for (var i = 0; i < child.length; i++) {
                    if (child[i]['groupIndex'] === treemap.initialDrillDown.groupIndex &&
                        child[i]['name'] === treemap.initialDrillDown.groupName) {
                        child[i]['isDrilled'] = true;
                        newData[child[i]['groupName']] = [child[i]];
                        drillData.push(newData);
                    }
                }
                for (var j = 0; j < child.length; j++) {
                    this.getDrilldownData(child[j], drillData);
                }
            }
            return drillData;
        };
        LayoutPanel.prototype.calculateLayoutItems = function (data, rect) {
            this.renderItems = [];
            this.parentData = [];
            if (!ej2_base_1.isNullOrUndefined(this.treemap.weightValuePath)) {
                if (this.treemap.layoutType.indexOf('SliceAndDice') > -1) {
                    this.computeSliceAndDiceDimensional(data, rect);
                }
                else {
                    rect.height = rect.height + rect.y;
                    rect.width = rect.width + rect.x;
                    this.computeSquarifyDimensional(data, rect);
                }
            }
        };
        LayoutPanel.prototype.computeSliceAndDiceDimensional = function (data, coords) {
            var leafItem = this.treemap.leafItemSettings;
            var rect;
            var groups = this.treemap.levels;
            var groupIndex;
            var isLeafItem = false;
            var children = helper_2.findChildren(data)['values'];
            var gap;
            var headerHeight;
            if (children && children.length > 0) {
                this.sliceAndDiceProcess(children, coords);
                if (this.treemap.levels.length > 0) {
                    for (var i = 0; i < children.length; i++) {
                        groupIndex = children[i]['groupIndex'];
                        isLeafItem = (groups.length === 0 || groupIndex === groups.length);
                        gap = isLeafItem ? leafItem.gap : groups[groupIndex].groupGap;
                        headerHeight = groups.length === 0 ? 0 : groups[groupIndex] ? groups[groupIndex].showHeader ?
                            groups[groupIndex].headerHeight : 0 : groups[groupIndex - 1].showHeader ? groups[groupIndex - 1].headerHeight : 0;
                        rect = children[i]['rect'];
                        rect = new helper_1.Rect(rect.x + (gap / 2), rect.y + (headerHeight + (gap / 2)), rect.width - gap, Math.abs(rect.height - (gap + headerHeight)));
                        this.computeSliceAndDiceDimensional(children[i], rect);
                    }
                }
            }
            return data;
        };
        LayoutPanel.prototype.sliceAndDiceProcess = function (processData, rect) {
            var parentArea = rect.height * rect.width;
            var levels = this.treemap.levels;
            var childValue;
            var alottedValue = 0;
            var totalWeight = 0;
            processData.forEach(function (data) { totalWeight += data['weight']; });
            processData.forEach(function (child) {
                child['weightArea'] = parentArea * child['weight'] / totalWeight;
            });
            var isHorizontal = (this.treemap.layoutType === 'SliceAndDiceAuto') ? (rect.width > rect.height) :
                (this.treemap.layoutType === 'SliceAndDiceHorizontal');
            processData.sort(helper_1.itemsToOrder);
            for (var i = 0; i < processData.length; i++) {
                var item = processData[i];
                item['isLeafItem'] = (levels.length === 0) || ((this.treemap.isHierarchicalData ||
                    ej2_base_1.isNullOrUndefined(this.treemap.leafItemSettings.labelPath)) ?
                    item['groupIndex'] === levels.length - 1 : item['groupIndex'] === this.treemap.levels.length);
                if (isHorizontal) {
                    childValue = ((parentArea / totalWeight) * processData[i]['weight']) / rect.height;
                    if (alottedValue <= rect.width) {
                        processData[i]['rect'] = new helper_1.Rect(alottedValue + rect.x, rect.y, childValue, rect.height);
                    }
                }
                else {
                    childValue = ((parentArea / totalWeight) * processData[i]['weight']) / rect.width;
                    if (alottedValue <= rect.height) {
                        processData[i]['rect'] = new helper_1.Rect(rect.x, alottedValue + rect.y, rect.width, childValue);
                    }
                }
                alottedValue += childValue;
                this.renderItems.push(processData[i]);
            }
        };
        LayoutPanel.prototype.computeSquarifyDimensional = function (data, coords) {
            var leaf = this.treemap.leafItemSettings;
            var rect;
            var levels = this.treemap.levels;
            var groupIndex;
            var isLeafItem = false;
            var item;
            var child = helper_2.findChildren(data)['values'];
            var index;
            var gap;
            var padding;
            var headerHeight;
            if (child && child.length > 0) {
                if (this.parentData.length === 0) {
                    this.parentData = [];
                    this.parentData.push(child);
                }
                this.calculateChildrenLayout(data, child, coords);
                if (this.treemap.levels.length > 0) {
                    for (var i = 0; i < child.length; i++) {
                        item = child[i];
                        index = item['groupIndex'];
                        rect = item['rect'];
                        gap = (item['isLeafItem'] ? leaf.gap : levels[index].groupGap) / 2;
                        padding = (item['isLeafItem'] ? leaf.padding : levels[index].groupPadding) / 2;
                        headerHeight = this.treemap.isHierarchicalData ? index === 0 && item['isLeafItem'] ? 0 : levels[index] ?
                            levels[index].showHeader ? levels[index].headerHeight : 0 : 0 : (levels.length === 0) ? 0 : levels[index] ?
                            levels[index].showHeader ? levels[index].headerHeight : 0 : 0;
                        rect = new helper_1.Rect(rect.x + padding, rect.y + (headerHeight + padding), rect.width - padding, rect.height - padding);
                        if (!item['isLeafItem'] && item['weight'] > 0) {
                            this.computeSquarifyDimensional(child[i], rect);
                        }
                    }
                }
            }
        };
        LayoutPanel.prototype.calculateChildrenLayout = function (parent, children, coords) {
            this.computeTotalArea(children, helper_1.getArea(coords));
            children.sort(helper_1.orderByArea);
            this.performRowsLayout(children, [], coords, []);
        };
        LayoutPanel.prototype.performRowsLayout = function (data, currentRow, rect, stack) {
            var dataLength = data.length;
            if (dataLength === 0) {
                var newCoordinates = this.getCoordinates(currentRow, rect);
                var newStack = stack.concat(newCoordinates);
                return newStack;
            }
            var width = helper_1.getShortestEdge(rect);
            var nextDatum = data[0];
            var restData = data.slice(1, dataLength);
            if (this.aspectRatio(currentRow, nextDatum, width)) {
                var newRow = currentRow.concat(nextDatum);
                return this.performRowsLayout(restData, newRow, rect, stack);
            }
            else {
                var currentRowLength = currentRow.length;
                var valueSum = 0;
                for (var i = 0; i < currentRowLength; i += 1) {
                    valueSum += currentRow[i]['itemArea'];
                }
                var newContainer = this.cutArea(rect, valueSum);
                var newCoordinates = this.getCoordinates(currentRow, rect);
                var newStack = stack.concat(newCoordinates);
                return this.performRowsLayout(data, [], newContainer, newStack);
            }
        };
        LayoutPanel.prototype.aspectRatio = function (currentRow, nextDatum, length) {
            if (currentRow.length === 0) {
                return true;
            }
            else {
                var newRow = currentRow.concat(nextDatum);
                var currentMaxAspectRatio = this.findMaxAspectRatio(currentRow, length);
                var newMaxAspectRatio = this.findMaxAspectRatio(newRow, length);
                return (currentMaxAspectRatio >= newMaxAspectRatio);
            }
        };
        LayoutPanel.prototype.findMaxAspectRatio = function (row, length) {
            var rowLength = row.length;
            var minArea = Infinity;
            var maxArea = -Infinity;
            var sumArea = 0;
            for (var i = 0; i < rowLength; i += 1) {
                var area = row[i]['itemArea'];
                if (area < minArea) {
                    minArea = area;
                }
                if (area > maxArea) {
                    maxArea = area;
                }
                sumArea += area;
            }
            var result = Math.max((Math.pow(length, 2)) * maxArea / (Math.pow(sumArea, 2)), (Math.pow(sumArea, 2)) /
                ((Math.pow(length, 2)) * minArea));
            return result;
        };
        LayoutPanel.prototype.cutArea = function (rect, area) {
            var newContainer = helper_1.convertToContainer(rect);
            var width = newContainer.width;
            var height = newContainer.height;
            var xOffset = newContainer.x;
            var yOffset = newContainer.y;
            if (width >= height) {
                var areaWidth = area / height;
                var newWidth = width - areaWidth;
                var container = {
                    x: xOffset + areaWidth,
                    y: yOffset,
                    width: newWidth,
                    height: height,
                };
                return helper_1.convertToRect(container);
            }
            else {
                var areaHeight = area / width;
                var newHeight = height - areaHeight;
                var container = {
                    x: xOffset,
                    y: yOffset + areaHeight,
                    width: width,
                    height: newHeight,
                };
                return helper_1.convertToRect(container);
            }
        };
        LayoutPanel.prototype.getCoordinates = function (row, rect) {
            var container = helper_1.convertToContainer(rect);
            var headerHeight;
            var width = container.width;
            var height = container.height;
            var xOffset = container.x;
            var yOffset = container.y;
            var rowLength = row.length;
            var levels = this.treemap.levels;
            var leaf = this.treemap.leafItemSettings;
            var index;
            var valueSum = 0;
            for (var i = 0; i < rowLength; i += 1) {
                valueSum += row[i]['itemArea'];
            }
            var areaWidth = valueSum / height;
            var areaHeight = valueSum / width;
            var subXOffset = xOffset;
            var subYOffset = yOffset;
            var padding;
            var coordinates = [];
            var isParent;
            var gap;
            var parentRect;
            for (var i = 0; i < rowLength; i += 1) {
                var item = row[i];
                index = item['groupIndex'];
                item['isLeafItem'] = (levels.length === 0) || (this.treemap.isHierarchicalData ? index === levels.length :
                    ej2_base_1.isNullOrUndefined(leaf.labelPath) ? false : index === levels.length);
                isParent = helper_1.isParentItem(this.parentData[0], item);
                parentRect = isParent ? this.treemap.areaRect : item['parent'].rect;
                padding = item['isLeafItem'] ? leaf.padding : levels[index].groupPadding;
                if (width >= height) {
                    var y1 = subYOffset + item['itemArea'] / areaWidth;
                    item['rect'] = {
                        x: subXOffset,
                        y: subYOffset,
                        width: subXOffset + areaWidth,
                        height: y1,
                    };
                    subYOffset = y1;
                }
                else {
                    var x1 = subXOffset + item['itemArea'] / areaHeight;
                    item['rect'] = {
                        x: subXOffset,
                        y: subYOffset,
                        width: x1,
                        height: subYOffset + areaHeight,
                    };
                    subXOffset = x1;
                }
                if (item['weight'] > 0 && (isParent || (Math.round(rect.y + (padding / 2)) <=
                    Math.round(parentRect.y + (parentRect.height - parentRect.y)) && Math.round(rect.x + (padding / 2)) <=
                    Math.round(parentRect.x + (parentRect.width - parentRect.x))))) {
                    this.renderItems.push(item);
                    coordinates.push(item);
                }
            }
            return coordinates;
        };
        LayoutPanel.prototype.computeTotalArea = function (data, area) {
            var dataLength = data.length;
            var dataSum = 0;
            var result = [];
            for (var i = 0; i < dataLength; i += 1) {
                var dataLength_1 = data.length;
                var dataSum_1 = 0;
                for (var i_1 = 0; i_1 < dataLength_1; i_1 += 1) {
                    dataSum_1 += data[i_1]['weight'];
                }
                var multiplier = area / dataSum_1;
                var datum = void 0;
                for (var j = 0; j < dataLength_1; j++) {
                    datum = data[j];
                    datum['itemArea'] = datum['weight'] * multiplier;
                    result.push(datum);
                }
            }
            return result;
        };
        LayoutPanel.prototype.renderLayoutItems = function (renderData) {
            var textCollection = [];
            var position;
            var treeMap = this.treemap;
            var colorMapping;
            var txtVisible;
            var getItemColor;
            var eventArgs;
            this.renderer = treeMap.renderer;
            var trimHeader;
            var textLocation = new helper_2.Location(0, 0);
            var pathOptions;
            var elementID = treeMap.element.id;
            var index;
            var templatePosition;
            var mode = treeMap.layoutType;
            var rect;
            var format;
            var interSectAction = this.treemap.leafItemSettings.interSectAction;
            var textSize;
            var fill;
            var item;
            var renderText;
            var opacity;
            var padding = 5;
            var rectPath = '';
            var isRender;
            var secondaryEle = document.getElementById(treeMap.element.id + '_Secondary_Element');
            var groupId;
            var textOptions;
            var templateEle;
            var gap;
            var textStyle;
            var levels = treeMap.levels;
            this.layoutGroup = this.renderer.createGroup({ id: elementID + '_TreeMap_' + mode + '_Layout' });
            var itemGroup;
            var level;
            var template;
            var border;
            var templateGroup = ej2_base_1.createElement('div', {
                id: treeMap.element.id + '_Label_Template_Group',
                className: 'template',
                styles: 'overflow: hidden; position: absolute;pointer-events: none;' +
                    'top:' + treeMap.areaRect.y + 'px;' +
                    'left:' + treeMap.areaRect.x + 'px;' +
                    'height:' + treeMap.areaRect.height + 'px;' +
                    'width:' + treeMap.areaRect.width + 'px;'
            });
            var isLeafItem = false;
            var leaf = treeMap.leafItemSettings;
            var childItems;
            for (var i = 0; i < this.renderItems.length; i++) {
                item = this.renderItems[i];
                index = item['groupIndex'];
                rect = item['rect'];
                isLeafItem = item['isLeafItem'];
                groupId = elementID + '_Level_Index_' + index + '_Item_Index_' + i;
                itemGroup = this.renderer.createGroup({ id: groupId + '_Group' });
                gap = (isLeafItem ? leaf.gap : levels[index].groupGap) / 2;
                if (treeMap.layoutType === 'Squarified') {
                    rect.width = Math.abs(rect.x - rect.width) - gap;
                    rect.height = Math.abs(rect.y - rect.height) - gap;
                }
                colorMapping = isLeafItem ? leaf.colorMapping : levels[index].colorMapping;
                getItemColor = this.getItemColor(isLeafItem, item);
                fill = getItemColor['fill'];
                opacity = getItemColor['opacity'];
                format = isLeafItem ? leaf.labelFormat : (levels[index]).headerFormat;
                txtVisible = isLeafItem ? leaf.showLabels : (levels[index]).showHeader;
                renderText = helper_1.textFormatter(format, item['data'], this.treemap) || item['name'];
                childItems = helper_2.findChildren(item)['values'];
                renderText = !isLeafItem && childItems && childItems.length > 0 && this.treemap.enableDrillDown ?
                    !item['isDrilled'] ? '[+] ' + renderText : '[-] ' + renderText : renderText;
                textStyle = (isLeafItem ? leaf.labelStyle : levels[index].headerStyle);
                border = isLeafItem ? leaf.border : levels[index].border;
                position = !isLeafItem ? (levels[index].headerAlignment) === 'Near' ? 'TopLeft' : (levels[index].headerAlignment) === 'Center' ?
                    'TopCenter' : 'TopRight' : leaf.labelPosition;
                templatePosition = isLeafItem ? leaf.templatePosition : levels[index].templatePosition;
                template = isLeafItem ? leaf.labelTemplate : levels[index].headerTemplate;
                item['options'] = { border: border, opacity: opacity, fill: fill };
                eventArgs = {
                    cancel: false, name: constants_1.itemRendering, treemap: this.treemap,
                    currentItem: item, RenderItems: this.renderItems, options: item['options']
                };
                this.treemap.trigger(constants_1.itemRendering, eventArgs);
                if (!eventArgs.cancel) {
                    rectPath = ' M ' + rect.x + ' ' + rect.y + ' L ' + (rect.x + rect.width) + ' ' + rect.y +
                        ' L ' + (rect.x + rect.width) + ' ' + (rect.y + rect.height) + ' L ' + rect.x + ' ' + (rect.y + rect.height) + 'z';
                    pathOptions = new helper_1.PathOption(groupId + '_RectPath', fill, border.width, border.color, opacity, null, rectPath);
                    var path = this.renderer.drawPath(pathOptions);
                    itemGroup.appendChild(path);
                    if (txtVisible) {
                        this.renderItemText(renderText.toString(), itemGroup, textStyle, rect, interSectAction, groupId, fill, position);
                    }
                    if (template) {
                        templateEle = this.renderTemplate(secondaryEle, groupId, rect, templatePosition, template, item);
                        templateGroup.appendChild(templateEle);
                    }
                    itemGroup.setAttribute('aria-label', item['name']);
                    itemGroup.setAttribute('tabindex', (this.treemap.tabIndex + i + 2).toString());
                    this.layoutGroup.appendChild(itemGroup);
                }
            }
            if (templateGroup.childNodes.length > 0) {
                secondaryEle.appendChild(templateGroup);
            }
            this.treemap.svgObject.appendChild(this.layoutGroup);
        };
        LayoutPanel.prototype.renderItemText = function (text, parentElement, textStyle, rect, interSectAction, groupId, fill, position) {
            var level;
            var textOptions;
            var headerPosition;
            var secondaryEle = document.getElementById(this.treemap.element.id + '_Secondary_Element');
            var leaf = this.treemap.leafItemSettings;
            var padding = 5;
            var textSize;
            var textLocation;
            var textCollection = [];
            var customText;
            var templateEle;
            var tspanText = [];
            var height = 0;
            var textName;
            textCollection = ((text.indexOf('<br>')) !== -1) ? text.split('<br>') : null;
            customText = this.labelInterSectAction(rect, text, textStyle, interSectAction);
            textSize = helper_1.measureText(textCollection && textCollection[0] || customText[0], textStyle);
            textLocation = helper_1.findLabelLocation(rect, position, textSize, 'Text', this.treemap);
            if (!ej2_base_1.isNullOrUndefined(textCollection)) {
                var collection = [];
                var texts = null;
                var maxNumber = [];
                for (var i = 0; i < textCollection.length; i++) {
                    texts = helper_1.textTrim((rect.width - 5), textCollection[i], textStyle);
                    textSize = helper_1.measureText(texts, textStyle);
                    height += textSize.height;
                    maxNumber.push(textSize.width);
                    collection.push(texts);
                }
                customText = collection;
                textSize.width = Math.max.apply(null, maxNumber);
                textSize.height = height;
            }
            if (interSectAction === 'WrapByWord' || interSectAction === 'Wrap' || interSectAction === 'Trim') {
                for (var j = 0; j < customText.length; j++) {
                    textSize = helper_1.measureText(customText[j], textStyle);
                    height += textSize.height;
                    if ((rect.height - padding) > height) {
                        tspanText.push(customText[j]);
                    }
                }
                if (interSectAction === 'Wrap' && customText.length !== tspanText.length && tspanText.length) {
                    var collectionLength = tspanText.length - 1;
                    var stringText = tspanText[collectionLength];
                    stringText = stringText.substring(0, (stringText.length - 1)) + '...';
                    tspanText.splice(collectionLength);
                    if (stringText !== '...') {
                        tspanText.push(stringText);
                    }
                }
            }
            else {
                textName = customText;
                tspanText.push(textName);
            }
            textOptions = new helper_1.TextOption(groupId + '_Text', textLocation.x, textLocation.y, 'start', tspanText);
            helper_2.renderTextElement(textOptions, textStyle, textStyle.color || this.getSaturatedColor(fill), parentElement);
        };
        LayoutPanel.prototype.getItemColor = function (isLeafItem, item) {
            var treemap = this.treemap;
            var itemFill = isLeafItem ? treemap.leafItemSettings.fill : treemap.levels[item['groupIndex']].fill;
            var itemOpacity = isLeafItem ? treemap.leafItemSettings.opacity : treemap.levels[item['groupIndex']].opacity;
            var parentData = helper_2.findChildren(treemap.levelsOfData[0])['values'];
            var colorMapping = isLeafItem ? treemap.leafItemSettings.colorMapping :
                treemap.levels[item['groupIndex']].colorMapping;
            if (colorMapping.length > 0) {
                var option = helper_1.colorMap(colorMapping, item['data'][this.treemap.equalColorValuePath], item['data'][this.treemap.rangeColorValuePath], item['data'][this.treemap.weightValuePath]);
                itemFill = option['fill'];
                itemOpacity = option['opacity'];
            }
            else {
                for (var i = 0; i < parentData.length; i++) {
                    if (parentData[i]['levelOrderName'] === item['levelOrderName'].split('_')[0]) {
                        itemFill = treemap.palette.length > 0 ? treemap.palette[i % treemap.palette.length] : itemFill;
                    }
                }
            }
            return { fill: itemFill, opacity: itemOpacity };
        };
        LayoutPanel.prototype.getSaturatedColor = function (color) {
            var saturatedColor = color;
            saturatedColor = (saturatedColor === 'transparent') ? window.getComputedStyle(document.body, null).backgroundColor : saturatedColor;
            var rgbValue = helper_1.convertHexToColor(helper_1.colorNameToHex(saturatedColor));
            var contrast = Math.round((rgbValue.r * 299 + rgbValue.g * 587 + rgbValue.b * 114) / 1000);
            return contrast >= 128 ? 'black' : 'white';
        };
        LayoutPanel.prototype.renderTemplate = function (secondaryEle, groupId, rect, position, template, item) {
            var templateElement;
            var labelEle;
            var templateSize;
            var templateFn;
            var templateLocation;
            var keys = Object.keys(item['data']);
            for (var i = 0; i < keys.length; i++) {
                template = template.replace(new RegExp('{{:' + keys[i] + '}}', 'g'), item['data'][keys[i].toString()]);
            }
            templateFn = helper_1.getTemplateFunction(template);
            templateElement = templateFn(this.treemap);
            labelEle = helper_1.convertElement(templateElement, groupId + '_Template', item['data']);
            templateSize = helper_1.measureElement(labelEle, secondaryEle);
            templateLocation = helper_1.findLabelLocation(rect, position, templateSize, 'Template', this.treemap);
            labelEle.style.left = templateLocation.x + 'px';
            labelEle.style.top = templateLocation.y + 'px';
            return labelEle;
        };
        LayoutPanel.prototype.labelInterSectAction = function (rect, text, textStyle, alignment) {
            var textValue;
            var maxWidth = rect.width - 10;
            switch (alignment) {
                case 'Hide':
                    textValue = [helper_1.hide(maxWidth, rect.height, text, textStyle)];
                    break;
                case 'Trim':
                    textValue = [helper_1.textTrim((maxWidth + 3), text, textStyle)];
                    break;
                case 'WrapByWord':
                    textValue = helper_1.wordWrap(maxWidth, text, textStyle);
                    break;
                case 'Wrap':
                    textValue = helper_1.textWrap(maxWidth, text, textStyle);
                    break;
            }
            return textValue;
        };
        return LayoutPanel;
    }());
    exports.LayoutPanel = LayoutPanel;
});
