define(["require", "exports", "@syncfusion/ej2-base", "../model/constants", "../utils/helper"], function (require, exports, ej2_base_1, constants_1, helper_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var TreeMapHighlight = (function () {
        function TreeMapHighlight(treeMap) {
            this.treemap = treeMap;
            this.addEventListener();
        }
        TreeMapHighlight.prototype.mouseMove = function (e) {
            var treemap = this.treemap;
            var processHighlight;
            var targetId = e.target.id;
            var eventArgs;
            var items = [];
            var highlight = this.treemap.highlightSettings;
            var item;
            var highLightElements = [];
            var process;
            var treeMapElement;
            var element;
            var orders;
            if (targetId.indexOf('_Item_Index') > -1) {
                if (this.highLightId !== targetId) {
                    treeMapElement = document.getElementById(treemap.element.id + '_TreeMap_' + treemap.layoutType + '_Layout');
                    var selectionElements = document.getElementsByClassName('treeMapSelection');
                    item = this.treemap.layout.renderItems[parseFloat(targetId.split('_')[6])];
                    orders = helper_1.findHightLightItems(item, [], highlight.mode, treemap);
                    for (var i = 0; i < treeMapElement.childElementCount; i++) {
                        element = treeMapElement.childNodes[i];
                        process = true;
                        item = treemap.layout.renderItems[element.id.split('_')[6]];
                        for (var j = 0; j < selectionElements.length; j++) {
                            if (element.id === selectionElements[j].id) {
                                process = false;
                                break;
                            }
                        }
                        if (orders.indexOf(item['levelOrderName']) > -1 && process) {
                            highLightElements.push(element);
                            items.push(item);
                        }
                    }
                    helper_1.removeClassNames(document.getElementsByClassName('treeMapHighLight'), 'treeMapHighLight', treemap);
                    for (var k = 0; k < highLightElements.length; k++) {
                        element = highLightElements[k];
                        helper_1.applyOptions(element.childNodes[0], { border: highlight.border, fill: highlight.fill, opacity: highlight.opacity });
                        element.classList.add('treeMapHighLight');
                        this.highLightId = targetId;
                    }
                    eventArgs = { cancel: false, name: constants_1.itemHighlight, treemap: treemap, items: items, elements: highLightElements };
                    treemap.trigger(constants_1.itemHighlight, eventArgs);
                }
                else {
                    processHighlight = false;
                }
            }
            else {
                helper_1.removeClassNames(document.getElementsByClassName('treeMapHighLight'), 'treeMapHighLight', treemap);
                processHighlight = false;
            }
            return processHighlight;
        };
        TreeMapHighlight.prototype.addEventListener = function () {
            if (this.treemap.isDestroyed) {
                return;
            }
            this.treemap.on(ej2_base_1.Browser.touchMoveEvent, this.mouseMove, this);
        };
        TreeMapHighlight.prototype.removeEventListener = function () {
            if (this.treemap.isDestroyed) {
                return;
            }
            this.treemap.off(ej2_base_1.Browser.touchMoveEvent, this.mouseMove);
        };
        TreeMapHighlight.prototype.getModuleName = function () {
            return 'treeMapHighlight';
        };
        TreeMapHighlight.prototype.destroy = function (treeMap) {
            this.removeEventListener();
        };
        return TreeMapHighlight;
    }());
    exports.TreeMapHighlight = TreeMapHighlight;
    var TreeMapSelection = (function () {
        function TreeMapSelection(treeMap) {
            this.treemap = treeMap;
            this.addEventListener();
        }
        TreeMapSelection.prototype.mouseDown = function (e) {
            var targetEle = e.target;
            var eventArgs;
            var treemap = this.treemap;
            var items = [];
            var targetId = targetEle.id;
            var item;
            var selectionElements = [];
            var opacity;
            var treeMapElement;
            var element;
            var orders;
            var selection = treemap.selectionSettings;
            var layoutID = treemap.element.id + '_TreeMap_' + treemap.layoutType + '_Layout';
            if (targetId.indexOf('_Item_Index') > -1) {
                e.preventDefault();
                if (this.selectionId !== targetId) {
                    treeMapElement = document.getElementById(layoutID);
                    item = treemap.layout.renderItems[parseFloat(targetId.split('_')[6])];
                    orders = helper_1.findHightLightItems(item, [], selection.mode, treemap);
                    for (var i = 0; i < treeMapElement.childElementCount; i++) {
                        element = treeMapElement.childNodes[i];
                        item = treemap.layout.renderItems[element.id.split('_')[6]];
                        if (orders.indexOf(item['levelOrderName']) > -1) {
                            selectionElements.push(element);
                            items.push(item);
                        }
                    }
                    helper_1.removeClassNames(document.getElementsByClassName('treeMapSelection'), 'treeMapSelection', treemap);
                    this.selectionId = targetId;
                    var highLightElements = document.getElementsByClassName('treeMapHighLight');
                    for (var k = 0; k < selectionElements.length; k++) {
                        element = selectionElements[k];
                        if (highLightElements.length > 0) {
                            for (var j = 0; j < highLightElements.length; j++) {
                                if (highLightElements[j].id === element.id) {
                                    highLightElements[j].classList.remove('treeMapHighLight');
                                }
                                helper_1.applyOptions(element.childNodes[0], { border: selection.border, fill: selection.fill, opacity: selection.opacity });
                                element.classList.add('treeMapSelection');
                            }
                        }
                        else {
                            helper_1.applyOptions(element.childNodes[0], { border: selection.border, fill: selection.fill, opacity: selection.opacity });
                            element.classList.add('treeMapSelection');
                        }
                        eventArgs = { cancel: false, name: constants_1.itemSelected, treemap: treemap, items: items, elements: selectionElements };
                        treemap.trigger(constants_1.itemSelected, eventArgs);
                    }
                }
                else {
                    helper_1.removeClassNames(document.getElementsByClassName('treeMapSelection'), 'treeMapSelection', treemap);
                    this.selectionId = '';
                }
            }
            else {
                helper_1.removeClassNames(document.getElementsByClassName('treeMapSelection'), 'treeMapSelection', treemap);
                this.selectionId = '';
            }
        };
        TreeMapSelection.prototype.addEventListener = function () {
            if (this.treemap.isDestroyed) {
                return;
            }
            this.treemap.on(ej2_base_1.Browser.touchStartEvent, this.mouseDown, this);
        };
        TreeMapSelection.prototype.removeEventListener = function () {
            if (this.treemap.isDestroyed) {
                return;
            }
            this.treemap.off(ej2_base_1.Browser.touchStartEvent, this.mouseDown);
        };
        TreeMapSelection.prototype.getModuleName = function () {
            return 'treeMapSelection';
        };
        TreeMapSelection.prototype.destroy = function (treeMap) {
            this.removeEventListener();
        };
        return TreeMapSelection;
    }());
    exports.TreeMapSelection = TreeMapSelection;
});
